#
# Copyright (C) 2015-2022  CZ.NIC, z. s. p. o.
#
# This file is part of FRED.
#
# FRED is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# FRED is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with FRED.  If not, see <https://www.gnu.org/licenses/>.
#
import warnings
from functools import lru_cache
from typing import Any, Dict, Optional

from django.core.cache import cache
from django.utils.functional import lazy
from django.utils.html import escape
from django.utils.safestring import mark_safe
from django.utils.translation import get_language, gettext_lazy as _
from django.views.generic.base import ContextMixin

from webwhois.utils import LOGGER

from ..constants import STATUS_DELETE_CANDIDATE, LogEntryType, LogResult
from ..exceptions import WebwhoisError

mark_safe_lazy = lazy(mark_safe, str)


class BaseContextMixin(ContextMixin):
    """Base mixin for webwhois views.

    @cvar base_template: Path to base template.
    """

    base_template = "base_site_example.html"

    def get_context_data(self, **kwargs):
        kwargs.setdefault("base_template", self.base_template)
        return super(BaseContextMixin, self).get_context_data(**kwargs)

    def render_to_response(self, context, **response_kwargs):
        self.request.current_app = self.request.resolver_match.namespace
        return super(BaseContextMixin, self).render_to_response(context, **response_kwargs)


class RegistryObjectMixin(BaseContextMixin):
    """Base class for loading object registry of the handle.

    It is rasied standard HTTP 500 "Server Error" page when Corba backend falied.
    Catch omniORB.CORBA.TRANSIENT and omniORB.CORBA.OBJECT_NOT_EXIST
    and redirect to your own customized page if you need.
    """

    _registry_objects_key = "registry_objects"
    _registry_objects_cache: Optional[Dict] = None

    server_exception_template = "webwhois/server_exception.html"
    object_type_name = None  # type: str
    log_entry_type = LogEntryType.INFO

    @staticmethod
    def _get_status_descriptions(type_name, fnc_get_descriptions):
        """Get status descritions from the cache. Load them from a backend and put in the cache if they missing there.

        Load status only for a defined object type and current site language.
        """
        lang = get_language()
        cache_key = "webwhois_descr_%s_%s" % (lang, type_name)
        descripts = cache.get(cache_key)
        if not descripts:
            descripts = {object_status_desc.handle: object_status_desc.name
                         for object_status_desc in fnc_get_descriptions(lang)}
            cache.set(cache_key, descripts)
        return descripts

    @staticmethod
    def message_with_handle_in_html(text, handle):
        """Make html message and mark it safe."""
        return mark_safe_lazy(text % "<strong>%s</strong>" % escape(handle))

    @classmethod
    def message_invalid_handle(cls, handle, code="INVALID_HANDLE"):
        return {
            "code": code,
            "title": _("Invalid handle"),
            "message": cls.message_with_handle_in_html(_("%s is not a valid handle."), handle),
        }

    @classmethod
    def load_registry_object(cls, context, handle):
        """Load registry object of the handle and append it into the context.

        Raises:
            WebwhoisError: If an expected error is returned from registry backend.
        """

    def load_related_objects(self, context):
        """Load objects related to the main registry object and append them into the context."""

    def _get_registry_objects(self):
        """Return a dict with objects loaded from the registry."""
        if self._registry_objects_cache is None:
            context = {self._registry_objects_key: {}}  # type: Dict[str, Any]
            properties = {"handle": self.kwargs["handle"], "handleType": self.object_type_name}
            with LOGGER.create(self.log_entry_type, source_ip=self.request.META.get('REMOTE_ADDR', ''),
                               properties=properties) as log_entry:
                try:
                    default_load = self.load_registry_object.__func__  # type: ignore[attr-defined]
                    if default_load.__module__.startswith('webwhois.views'):
                        obj = self.get_object()
                        context[self._registry_objects_key] = self._make_context(obj)
                    else:
                        warnings.warn(
                            "Method load_registry_object is deprecated, use get_object or get_context_data instead.",
                            DeprecationWarning)
                        self.load_registry_object(context, self.kwargs["handle"])

                except WebwhoisError as error:
                    context['server_exception'] = error
                except BaseException as error:
                    log_entry.result = LogResult.ERROR
                    log_entry.properties["exception"] = error.__class__.__name__
                    raise

                # It's here to handle `load_registry_object` results. Can be refactored, when removed.
                found_types = sorted(context.get(self._registry_objects_key, {}).keys())
                if len(found_types):
                    log_entry.result = LogResult.SUCCESS
                    log_entry.properties["foundType"] = found_types
                else:
                    log_entry.result = LogResult.NOT_FOUND
                    webwhois_error = context.get("server_exception")
                    if webwhois_error and webwhois_error.code != "OBJECT_NOT_FOUND":
                        log_entry.properties["reason"] = webwhois_error.code
            if len(context[self._registry_objects_key]) == 1:
                self.load_related_objects(context)
            self._registry_objects_cache = context
        return self._registry_objects_cache

    def _make_context(self, obj: Any) -> Dict[str, Any]:
        """Turn object into a context."""
        return {self.object_type_name: {"detail": obj}}

    @lru_cache()
    def get_object(self) -> Any:
        """Fetch and return an object from registry.

        Raises:
            WebwhoisError: If an expected error is returned from registry backend.
        """
        return self._get_object(self.kwargs['handle'])

    def _get_object(self, handle: str) -> Any:
        """Actually fetches and returns the object from registry.

        Arguments:
            handle: Handle of an object to find.

        Raises:
            WebwhoisError: If an expected error is returned from registry backend.
        """
        raise NotImplementedError

    def get_context_data(self, handle, **kwargs):
        kwargs.setdefault("handle", handle)
        objects = self._get_registry_objects()
        kwargs.update(objects)
        if self.object_type_name in objects[self._registry_objects_key]:
            obj = objects[self._registry_objects_key][self.object_type_name]
            # `obj` may be None, if domain is a delete candidate.
            if obj is not None:
                # Registrars don't have statuses
                kwargs['object_delete_candidate'] = STATUS_DELETE_CANDIDATE in getattr(obj['detail'], 'statuses', ())
        return super(RegistryObjectMixin, self).get_context_data(**kwargs)

    def get_template_names(self):
        context = self._get_registry_objects()
        if "server_exception" in context:
            return [self.server_exception_template]
        return super(RegistryObjectMixin, self).get_template_names()
